import { useRefById } from "svelte-toolbelt";
import { fromStore } from "svelte/store";
import { getContext, setContext } from "svelte";
import { extractErrorArray } from "./internal/utils/errors.js";
import { getValueAtPath } from "./internal/utils/path.js";
import { getAriaDescribedBy, getAriaInvalid, getAriaRequired, getDataFsError, } from "./internal/utils/attributes.js";
class FormFieldState {
    #form;
    #name;
    #formErrors;
    #formConstraints;
    #formTainted;
    #formData;
    name = $derived.by(() => this.#name.current);
    errors = $derived.by(() => extractErrorArray(getValueAtPath(this.#name.current, this.#formErrors.current)));
    constraints = $derived.by(() => getValueAtPath(this.#name.current, this.#formConstraints.current) ?? {});
    tainted = $derived.by(() => this.#formTainted.current
        ? getValueAtPath(this.#name.current, this.#formTainted.current) === true
        : false);
    errorNode = $state(null);
    descriptionNode = $state(null);
    constructor(props) {
        this.#form = props.form;
        this.#name = props.name;
        this.#formErrors = fromStore(props.form.current.errors);
        this.#formConstraints = fromStore(props.form.current.constraints);
        this.#formTainted = fromStore(props.form.current.tainted);
        this.#formData = fromStore(props.form.current.form);
    }
    snippetProps = $derived.by(() => ({
        value: this.#formData.current[this.#name.current],
        errors: this.errors,
        tainted: this.tainted,
        constraints: 
        // @ts-expect-error - this type is wonky
        this.#formConstraints.current[
        // eslint-disable-next-line ts/no-explicit-any
        this.#name.current] ?? {},
    }));
}
class ElementFieldState {
    #form;
    #name;
    #formErrors;
    #formConstraints;
    #formTainted;
    #formData;
    #field;
    name = $derived.by(() => {
        const [path] = splitArrayPath(this.#name.current);
        return path;
    });
    errors = $derived.by(() => extractErrorArray(getValueAtPath(this.#name.current, this.#formErrors.current)));
    constraints = $derived.by(() => getValueAtPath(this.#name.current, this.#formConstraints.current) ?? {});
    tainted = $derived.by(() => this.#formTainted.current
        ? getValueAtPath(this.#name.current, this.#formTainted.current) === true
        : false);
    errorNode = $state(null);
    descriptionNode = $state(null);
    // fall back to the parent field's description node if one for
    // this specific element doesn't exist.
    derivedDescriptionNode = $derived.by(() => {
        if (this.descriptionNode)
            return this.descriptionNode;
        if (this.#field.descriptionNode)
            return this.#field.descriptionNode;
        return null;
    });
    value = $derived.by(() => {
        return getValueAtPath(this.#name.current, this.#formData.current);
    });
    constructor(props, field) {
        this.#form = props.form;
        this.#name = props.name;
        this.#formErrors = fromStore(props.form.current.errors);
        this.#formConstraints = fromStore(props.form.current.constraints);
        this.#formTainted = fromStore(props.form.current.tainted);
        this.#formData = fromStore(props.form.current.form);
        this.#field = field;
    }
    snippetProps = $derived.by(() => ({
        value: this.#formData.current[this.#name.current],
        errors: this.errors,
        tainted: this.tainted,
        constraints: 
        // @ts-expect-error - this type is wonky
        this.#formConstraints.current[
        // eslint-disable-next-line ts/no-explicit-any
        this.#name.current] ?? {},
    }));
}
class FieldErrorsState {
    #ref;
    #id;
    field;
    #errorAttr = $derived.by(() => getDataFsError(this.field.errors));
    constructor(props, field) {
        this.#ref = props.ref;
        this.#id = props.id;
        this.field = field;
        useRefById({
            id: this.#id,
            ref: this.#ref,
            onRefChange: (node) => {
                this.field.errorNode = node;
            },
        });
    }
    snippetProps = $derived.by(() => ({
        errors: this.field.errors,
        errorProps: this.errorProps,
    }));
    fieldErrorsProps = $derived.by(() => ({
        id: this.#id.current,
        "data-fs-error": this.#errorAttr,
        "data-fs-field-errors": "",
        "aria-live": "assertive",
    }));
    errorProps = $derived.by(() => ({
        "data-fs-field-error": "",
        "data-fs-error": this.#errorAttr,
    }));
}
class DescriptionState {
    #ref;
    #id;
    field;
    constructor(props, field) {
        this.#ref = props.ref;
        this.#id = props.id;
        this.field = field;
        useRefById({
            id: this.#id,
            ref: this.#ref,
            onRefChange: (node) => {
                this.field.descriptionNode = node;
            },
        });
    }
    props = $derived.by(() => ({
        id: this.#id.current,
        "data-fs-error": getDataFsError(this.field.errors),
        "data-fs-description": "",
    }));
}
class ControlState {
    id;
    field;
    constructor(props, field) {
        this.id = props.id;
        this.field = field;
    }
    props = $derived.by(() => ({
        id: this.id.current,
        name: this.field.name,
        "data-fs-error": getDataFsError(this.field.errors),
        "aria-describedby": getAriaDescribedBy({
            fieldErrorsId: this.field.errorNode?.id,
            descriptionId: this.field.descriptionNode?.id,
            errors: this.field.errors,
        }),
        "aria-invalid": getAriaInvalid(this.field.errors),
        "aria-required": getAriaRequired(this.field.constraints),
        "data-fs-control": "",
    }));
}
class LabelState {
    #ref;
    #id;
    control;
    constructor(props, control) {
        this.#ref = props.ref;
        this.#id = props.id;
        this.control = control;
        useRefById({
            id: this.#id,
            ref: this.#ref,
        });
    }
    props = $derived.by(() => ({
        id: this.#id.current,
        "data-fs-label": "",
        "data-fs-error": getDataFsError(this.control.field.errors),
        for: this.control.id.current,
    }));
}
class LegendState {
    #ref;
    #id;
    field;
    constructor(props, field) {
        this.#ref = props.ref;
        this.#id = props.id;
        this.field = field;
        useRefById({
            id: this.#id,
            ref: this.#ref,
        });
    }
    props = $derived.by(() => ({
        id: this.#id.current,
        "data-fs-error": getDataFsError(this.field.errors),
        "data-fs-legend": "",
    }));
}
const FORM_FIELD_CTX = Symbol.for("formsnap.form-field");
const FORM_CONTROL_CTX = Symbol.for("formsnap.form-control");
export function useFormField(props) {
    return setContext(FORM_FIELD_CTX, new FormFieldState(props));
}
export function useElementField(props) {
    const formField = getFormField();
    return setContext(FORM_FIELD_CTX, new ElementFieldState(props, formField));
}
export function getFormField() {
    return getContext(FORM_FIELD_CTX);
}
export function useFieldErrors(props) {
    return new FieldErrorsState(props, getFormField());
}
export function useDescription(props) {
    return new DescriptionState(props, getFormField());
}
export function useControl(props) {
    return setContext(FORM_CONTROL_CTX, new ControlState(props, getFormField()));
}
export function getFormControl() {
    return getContext(FORM_CONTROL_CTX);
}
export function useLabel(props) {
    return new LabelState(props, getFormControl());
}
export function useLegend(props) {
    return new LegendState(props, getFormField());
}
// takes a string like "urls[0]" and returns ["urls", "0"]
// so we can access the specific array index properties
// since datatype: json is not supported with regular form
// submission, this should be fine
function splitArrayPath(name) {
    const [path, index] = name.split(/[[\]]/);
    return [path, index];
}
